#region Copyright Syncfusion Inc. 2001 - 2005
//
//  Copyright Syncfusion Inc. 2001 - 2005. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
//
#endregion

using System;
using System.Data;
using System.Collections;
using System.ComponentModel; 


namespace CustomGridCell
{
    public class TreeNodeState
    {
        static public int Opened = 0;
        static public int Closed = 1;
        static public int NoChildren = 2;
    }

    [Serializable]
    public class SampleData
    {
        private int _count;
        private int _indentLevel;
        private string[] _items;
        private int _state;

        public string[] Items
        {
            get
            {
                if (_items == null)
                    _items = new string[_count];
                return _items;
            }
            set { _items = value; }
        }


        public int IndentLevel
        {
            get { return _indentLevel; }
            set { _indentLevel = value; }
        }

        public int ItemCount
        {
            get { return _count; }
        }

        public int ExpandState
        {
            get { return _state; }
            set { _state = value; }
        }

        //public SampleData(int indentLevel, int expandstate, string item1, string item2, string item3, string item4)
        //{
        //    _count = 4;
        //    _items = new string[_count];
        //    _items[0] = item1;
        //    _items[1] = item2;
        //    _items[2] = item3;
        //    _items[3] = item4;
        //    _indentLevel = indentLevel;
        //    _state = expandstate;

        //}
        public SampleData(int indentLevel, int expandstate, string[] items)
        {
            _count = items.GetLength(0);
            _items = items;
            _indentLevel = indentLevel;
            _state = expandstate;

        }
    }

    [Serializable]
    public class CollapsibleDataSource
    {
        private int internalRowCount;
        private int internalColCount;
        private SampleData[] data;

        private ArrayList visibleRows;

        public bool Locked;

        public CollapsibleDataSource(int rawRowCount)
        {
            internalColCount = 4;

            Locked = false;

            internalRowCount = rawRowCount;

            visibleRows = new ArrayList();

        }

        public int RowCount
        {
            get { return visibleRows.Count; }
        }

        public int ColCount
        {
            get { return internalColCount; }
        }

        public void CollapseData(int start)
        {
            //decide which rows are visible and add then to visibleRows
            if (start > 0)
                visibleRows.RemoveRange(start, visibleRows.Count - start);
            else
                visibleRows = new ArrayList();

            int i = start;
            while (i < internalRowCount)
            {
                i = ProcessNode(i);// will increment i at least once, maybe more
            }
        }
        public void CollapseData()
        {
            //decide which rows are visible and add then to visibleRows
            visibleRows = new ArrayList();

            int i = 0;
            while (i < internalRowCount)
            {
                i = ProcessNode(i);// will increment i at least once, maybe more
            }
        }

        private int ProcessNode(int i)
        {
            if (i >= internalRowCount)
                return i;

            int indent = data[i].IndentLevel;
            bool closed = (data[i].ExpandState == TreeNodeState.Closed);
            visibleRows.Add(i);

            //make sure parent is not a nocheck
            int k;
            if (visibleRows.Count > 1)
            {
                k = (int)visibleRows[visibleRows.Count - 2];
                if (data[k].ExpandState == TreeNodeState.NoChildren
                    && indent > data[k].IndentLevel)
                {
                    data[k].ExpandState = TreeNodeState.Opened;
                }
            }

            ++i;
            if (closed)
            {
                // since last node was not open, 
                // ignore all nodes indented further
                while (i < internalRowCount && data[i].IndentLevel > indent)
                    ++i;
                return i; // return & process next node at same level
            }

            // last node was open, so process all on same level
            while (i < internalRowCount && data[i].IndentLevel == indent)
            {
                i = ProcessNode(i);
                return i;
            }

            return i;


        }
        public SampleData this[int i]
        {
            get
            {
                if (visibleRows.Count > i)
                    return data[(int)visibleRows[i]];
                return null;
            }
        }


        public void ExpandAll()
        {
            for (int i = 0; i < internalRowCount; ++i)
            {
                if (data[i].ExpandState == TreeNodeState.Closed)
                    data[i].ExpandState = TreeNodeState.Opened;
            }
        }

        public void CloseAll()
        {
            for (int i = 0; i < internalRowCount; ++i)
            {
                if (data[i].ExpandState == TreeNodeState.Opened)
                    data[i].ExpandState = TreeNodeState.Closed;
            }

        }
        public void InitFromProduct(Product product)
        {
            int rows = product.rowCount;
            int cols = 0;

            this.data = new SampleData[100];
            cols = product.GetType().GetFields().Length - 2; 
            int ncols = TypeDescriptor.GetProperties(product.childProducts).Count;  

            if (cols < ncols)
                cols = ncols;
            int row = 0;
            
            // This is for setting up the first row..
            string[] s = new string[cols];
            string[] s1 = new string[cols];
            s[0] = "Product Hierarchy";
            
            data[row] = new SampleData(0, TreeNodeState.Closed, s);

            // Row 1 for a record
            s    = new string[cols];
            s[0] = product.ProductName;
            s[1] = product.DType;
            s[2] = product.dMonth;
            s[3] = product.dWeek1;
            s[4] = product.dWeek1;
            s[5] = product.dWeek3;
            s[6] = product.dWeek4;

            // Row 2 for a record. 
            s[7] = product.MType;
            s[8] = product.mMonth;
            s[9] = product.mWeek1;
            s[10] = product.mWeek2;
            s[11] = product.mWeek3;
            s[12] = product.mWeek4;

            row++;
            data[row] = new SampleData(1, TreeNodeState.NoChildren, s);

            foreach (Product item in product.childProducts)
            {
                row++;
                s = new string[cols];
                s[0] = product.ProductName;
                s[1] = product.DType;
                s[2] = product.dMonth;
                s[3] = product.dWeek1;
                s[4] = product.dWeek1;
                s[5] = product.dWeek3;
                s[6] = product.dWeek4;

                s[7] = product.MType;
                s[8] = product.mMonth;
                s[9] = product.mWeek1;
                s[10] = product.mWeek2;
                s[11] = product.mWeek3;
                s[12] = product.mWeek4;
                data[row] = new SampleData(2, TreeNodeState.NoChildren, s);
                foreach (Product grandChildItem in item.childProducts)
                {

                    row++;
                    s1 = new string[cols];
                    s1[0] = product.ProductName;
                    s1[1] = product.DType;
                    s1[2] = product.dMonth;
                    s1[3] = product.dWeek1;
                    s1[4] = product.dWeek1;
                    s1[5] = product.dWeek3;
                    s1[6] = product.dWeek4;

                    s1[7] = product.MType;
                    s1[8] = product.mMonth;
                    s1[9] = product.mWeek1;
                    s1[10] = product.mWeek2;
                    s1[11] = product.mWeek3;
                    s1[12] = product.mWeek4;
                    
                    data[row] = new SampleData(3, TreeNodeState.NoChildren, s1);
                }
                
            }

            internalColCount = cols;

            Locked = false;

            internalRowCount = rows;

            visibleRows = new ArrayList();

        }



    }




}
